# flake8: noqa: F401
# ruff: noqa: F401

from galileo_core.schemas.protect.action import (
    Action,
    ActionResult,
    OverrideAction,
    PassthroughAction,
)
from galileo_core.schemas.protect.metric import (
    MetricComputation,
    MetricComputationStatus,
)
from galileo_core.schemas.protect.payload import Payload
from galileo_core.schemas.protect.request import Request
from galileo_core.schemas.protect.rule import Rule, RuleOperator
from galileo_core.schemas.protect.ruleset import Ruleset
from galileo_core.schemas.protect.stage import Stage

from galileo_protect.schemas.invoke import Response



from pydantic import BaseModel, ConfigDict, Field


class Response(BaseModel):
    text: str = Field(description="Text from the request after processing the rules.")

    model_config = ConfigDict(extra="allow")



# flake8: noqa: F401
# ruff: noqa: F401
from galileo_protect.invoke import invoke
from galileo_protect.project import create_project
from galileo_protect.stage import create_stage

__version__ = "0.2.0"



from datetime import timedelta
from typing import Dict, Optional, Sequence

from pydantic import UUID4
from requests import post

from galileo_protect.constants.routes import Routes
from galileo_protect.helpers.config import ProtectConfig
from galileo_protect.schemas import Payload, Request, Ruleset
from galileo_protect.schemas.invoke import Response


def invoke(
    payload: Payload,
    prioritized_rulesets: Optional[Sequence[Ruleset]] = None,
    project_id: Optional[UUID4] = None,
    stage_name: Optional[str] = None,
    stage_id: Optional[UUID4] = None,
    timeout: float = timedelta(minutes=5).total_seconds(),
    metadata: Optional[Dict[str, str]] = None,
    headers: Optional[Dict[str, str]] = None,
    config: Optional[ProtectConfig] = None,
) -> Response:
    protect_config: ProtectConfig = config or ProtectConfig.get()
    response_json = protect_config.api_client.request(
        post,
        Routes.invoke,
        json=Request(
            payload=payload,
            rulesets=prioritized_rulesets or [],
            project_id=project_id or protect_config.project_id,
            stage_name=stage_name or protect_config.stage_name,
            stage_id=stage_id or protect_config.stage_id,
            timeout=timeout,
            metadata=metadata,
            headers=headers,
        ).model_dump(mode="json"),
    )
    return Response.model_validate(response_json)



from typing import Optional

from galileo_core.helpers.config import GalileoConfig
from pydantic import UUID4


class ProtectConfig(GalileoConfig):
    # Config file for this project.
    config_filename: str = "protect-config.json"

    # Protect specific configuration.
    project_id: Optional[UUID4] = None
    stage_name: Optional[str] = None
    stage_id: Optional[UUID4] = None

    def reset(self) -> None:
        super().reset()
        self.project_id = None
        self.stage_name = None
        self.stage_id = None



from typing import Optional

from galileo_core.helpers.project import create_project as core_create_project
from galileo_core.schemas.core.project import (
    DEFAULT_PROJECT_NAME,
    CreateProjectRequest,
    ProjectResponse,
    ProjectType,
)

from galileo_protect.helpers.config import ProtectConfig


def create_project(name: str = DEFAULT_PROJECT_NAME, config: Optional[ProtectConfig] = None) -> ProjectResponse:
    config = config or ProtectConfig.get()
    project = core_create_project(CreateProjectRequest(name=name, type=ProjectType.protect), config)
    config.project_id = project.id
    config.write()
    return project



from types import SimpleNamespace

# We cannot use Enum here because `mypy` doesn't like it when we format those routes
# into strings. https://github.com/python/mypy/issues/15269
Routes = SimpleNamespace(invoke="protect/invoke", stages="projects/{project_id}/stages")



from typing import Optional

from galileo_core.utils.name import ts_name
from pydantic import UUID4
from requests import post

from galileo_protect.constants.routes import Routes
from galileo_protect.helpers.config import ProtectConfig
from galileo_protect.schemas import Action, Stage
from galileo_protect.schemas.stage import StageResponse


def create_stage(
    project_id: Optional[UUID4] = None,
    name: Optional[str] = None,
    description: Optional[str] = None,
    action: Optional[Action] = None,
    action_elabed: bool = False,
    config: Optional[ProtectConfig] = None,
) -> StageResponse:
    config = config or ProtectConfig.get()
    project_id = project_id or config.project_id
    if project_id is None:
        raise ValueError("Project ID must be provided to create a stage.")
    name = name or ts_name("stage")
    stage = StageResponse.model_validate(
        config.api_client.request(
            post,
            Routes.stages.format(project_id=project_id),
            json=Stage(
                name=name, project_id=project_id, description=description, action=action, action_enabled=action_elabed
            ).model_dump(mode="json"),
        )
    )
    config.project_id = project_id
    config.stage_id = stage.id
    config.stage_name = stage.name
    config.write()
    return stage
